//--------------------------------------------------------
// StarChild by Andreas Gustafsson (Shadow/Noice) (C) 2000
//--------------------------------------------------------


#include "stdafx.h"
#include "Texture.h"
#include "fetus.h"

#define NR_COMETS 64
#define NR_LINES 16
#define BALL_SIZE 1.25

#pragma warning(disable : 4244)

//----------------------------------------
// Usefull debugger thingy.
void DebugOut(const char *szFormat, ...);
//----------------------------------------

// Change your App Name.
char szAppName[] = "Skeleton";
HWND hwnd;

// Timer info.  Very useful for keeping animated stuff in sync on different computers.
extern float fTime, fDeltaTime;

// Current window dimensions.
extern GLsizei g_w, g_h;

// Use the demonstration texture.
CTexture gTexExample;
CTexture gTexExample2;
CTexture gTexExample3;
int updateFPS=0;
double curFPS=0;
float rotx[NR_COMETS],roty[NR_COMETS],rotz[NR_COMETS],pos[NR_COMETS],colr[NR_COMETS],colg[NR_COMETS],colb[NR_COMETS],spd[NR_COMETS],fade[NR_COMETS],fadespd[NR_COMETS];
float fetSize=0.0;
float curComets=0;

float random(float high)
{
    return((float)((double)rand()/32768)*high);
}

// Startup Stuff.
bool Init()	// Called right after the window is created, and OpenGL is initialized.
{
	int i;

	glViewport (0, 0, (GLsizei) g_w, (GLsizei) g_h); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	gluPerspective(55.0, (GLfloat) g_w/(GLfloat) g_h, 1.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef (0.0, 0.0, -3.0);
	glClearColor(0.0,0.0,0.0,0.0);
	glEnable(GL_DEPTH_TEST);

	gTexExample.LoadJPG( "space2.jpg" );
	gTexExample2.LoadJPG( "lensbw.jpg" );
	gTexExample3.LoadJPG( "blueenv.jpg" );

	for(i=0;i<NR_COMETS;i++)
	{
		rotx[i]=random(360);
		roty[i]=random(360);
		rotz[i]=random(360);
		pos[i]=random(2*PI);
		colr[i]=random(1.0);
		colg[i]=random(1.0);
		colb[i]=random(1.0);
		spd[i]=0.5f+random(1.0);
		fade[i]=0.0;
		fadespd[i]=0.15f*((float)i/(float)NR_COMETS);
	}
	// If everything went well, then return true.
	// Returning false will cause the program to quit right away.
	return true;
}

void Shutdown() // Called right after the window is destroyed.
{
}

void drawComet(float rx,float ry,float rz,float ps,float r,float g,float b)
{
	int i;

	//glDisable(GL_DEPTH_TEST);
	glDepthMask(0);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);
	glPushMatrix();
	glRotatef( rx, 1.f, 0, 0 );
	glRotatef( ry, 0,1.f, 0 );
	glRotatef( rz, 0, 0, 1.f );
	gTexExample2.Use();
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_LINE_STRIP);
	for(i=0;i<NR_LINES;i++)
	{
		glColor3f(r*(1-(float)i/(float)NR_LINES),g*(1-(float)i/(float)NR_LINES),b*(1-(float)i/(float)NR_LINES));	
		glVertex3f(BALL_SIZE*cos(ps-(float)i/(float)NR_LINES*1),BALL_SIZE*sin(ps-(float)i/(float)NR_LINES*1),0);
	}
	glEnd();
	glEnable(GL_TEXTURE_2D);
	glColor3f(r,g,b);
	glTranslatef(BALL_SIZE*cos(ps),BALL_SIZE*sin(ps),0);
	glScalef(0.10f,0.10f,0.10f);
	glRotatef( -rz, 0, 0, 1.f );
	glRotatef( -ry, 0,1.f, 0 );
	glRotatef( -rx, 1.f, 0, 0 );
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(-1,-1,0);
	glTexCoord2f(1,0);
	glVertex3f(1,-1,0);
	glTexCoord2f(1,1);
	glVertex3f(1,1,0);
	glTexCoord2f(0,1);
	glVertex3f(-1,1,0);
	glEnd();	
	glPopMatrix();
	glDisable(GL_BLEND);
	//glEnable(GL_DEPTH_TEST);
	glDepthMask(1);
}

// Draw all the scene related stuff.
void Render()
{
	int i;
	float x,y;
	char s[100];

	curFPS+=1.f/fDeltaTime;
	updateFPS++;
	if(updateFPS==20)
	{
		sprintf(s,"StarChild by Andreas Gustafsson (Shadow/Noice) (C) 2000   -=[ FPS:%.0f ]=-",curFPS/20.0f);
		SetWindowText(hwnd,s);
		updateFPS=0;
		curFPS=0;
	}


	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
	glPushMatrix();
	glRotatef(15.0f*sin(fTime*0.7), 1.f, 0, 0 );
	glRotatef(15.0f*sin(fTime*0.4), 0,1.f, 0 );
	glRotatef(fTime*15.0f, 0, 0, 1.f );
	glEnable( GL_TEXTURE_2D );
	glScalef(4,4,1);
	gTexExample.Use();
	glBegin(GL_QUADS);
	glColor3f(0.75f,0.75f,0.75f);
//	glColor3f(1,1,1);
	for(y=-8;y<8;y+=1)
	{
		for(x=-8;x<8;x+=1)
		{
			glTexCoord2f( (x+8)/16, (y+8)/16);
			glVertex3f(x/8,y/8,-2+sqrt((float)((x+0)*(x+0)+(y+0)*(y+0)))/4.0);
			glTexCoord2f( (x+9)/16, (y+8)/16);
			glVertex3f((x+1)/8,y/8,-2+sqrt((float)((x+1)*(x+1)+(y+0)*(y+0)))/4.0);
			glTexCoord2f( (x+9)/16, (y+9)/16);
			glVertex3f((x+1)/8,(y+1)/8,-2+sqrt((float)((x+1)*(x+1)+(y+1)*(y+1)))/4.0);
			glTexCoord2f( (x+8)/16, (y+9)/16);
			glVertex3f(x/8,(y+1)/8,-2+sqrt((float)((x+0)*(x+0)+(y+1)*(y+1)))/4.0);
		}
	}
	glEnd();
	glScalef(0.25,0.25,1);
	glRotatef( fTime*10,  1.f,0,0 );
	glRotatef( fTime*25, 0, 1.f ,0);
	glRotatef( fTime*10, 0, 0, 1.f );
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glTexGenf(GL_S,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
	glTexGenf(GL_T,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);
	gTexExample3.Use();
	glColor3f(0.5,0.5,0.5);
	glDepthMask(0);
	glScalef(0.75*fetSize,0.75*fetSize,0.75*fetSize);
	draw3DObj();	
	fetSize+=fDeltaTime*0.1;
	if(fetSize>1.0) fetSize=1.0;
	glDepthMask(1);
	glDisable(GL_TEXTURE_GEN_S);
	glDisable(GL_TEXTURE_GEN_T);
	glPopMatrix();
	for(i=0;i<NR_COMETS;i++) 
	{
		drawComet(rotx[i]-fTime*2,roty[i]+3,rotz[i]-5,pos[i],fade[i]*colr[i],fade[i]*colg[i],fade[i]*colb[i]);
		pos[i]+=fDeltaTime*spd[i];
		fade[i]+=fadespd[i]*fDeltaTime;
		if(fade[i]>1.0) fade[i]=1.0;
	}
	glPopMatrix();
}
